from datetime import datetime
from importlib.metadata import metadata

from django.utils import timezone
from django_rq import get_queue

from core.choices import JobStatusChoices
from netbox.plugins import PluginConfig
from netbox_config_backup.utils.logger import get_logger

metadata = metadata('netbox_config_backup')


class NetboxConfigBackup(PluginConfig):
    name = metadata.get('Name').replace('-', '_')
    verbose_name = metadata.get('Name').replace('-', ' ').title()
    description = metadata.get('Summary')
    version = metadata.get('Version')
    author = metadata.get('Author')
    author_email = metadata.get('Author-email')
    base_url = 'configbackup'
    min_version = '4.1.0'
    max_version = '4.2.99'
    required_settings = [
        'repository',
        'committer',
        'author',
    ]
    default_settings = {
        # Frequency in seconds
        'frequency': 3600,
    }
    queues = [
        'jobs'
    ]
    graphql_schema = 'graphql.schema.schema'

    def ready(self, *args, **kwargs):
        super().ready()
        import sys
        if len(sys.argv) > 1 and 'rqworker' in sys.argv[1]:
            from netbox import settings
            from netbox_config_backup.jobs.backup import BackupRunner
            frequency = settings.PLUGINS_CONFIG.get('netbox_config_backup', {}).get('frequency') / 60
            lastjob = BackupRunner.get_jobs().order_by('pk').last()

            if not lastjob:
                BackupRunner.enqueue_once(interval=frequency)
            elif lastjob.status in JobStatusChoices.ENQUEUED_STATE_CHOICES and lastjob.scheduled < timezone.now():
                BackupRunner.enqueue_once(interval=frequency)
            elif lastjob.status in JobStatusChoices.TERMINAL_STATE_CHOICES:
                scheduled = lastjob.created + timezone.timedelta(minutes=frequency)
                if scheduled < timezone.now():
                    scheduled = None
                BackupRunner.enqueue_once(interval=frequency, schedule_at=scheduled)



config = NetboxConfigBackup



from django.core.management.base import BaseCommand

from netbox_config_backup.jobs.backup import BackupRunner
from netbox_config_backup.models import Backup


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--time', dest='time', help="time")
        parser.add_argument('--device', dest='device', help="Device Name")

    def run_backup(self, backup=None):
        BackupRunner.enqueue(backup=backup, immediate=True)

    def handle(self, *args, **options):
        if options['device']:
            print(f'Running backup for: {options.get("device")}')
            backup = Backup.objects.filter(device__name=options['device']).first()
            if not backup:
                backup = Backup.objects.filter(name=options['device']).first()
            if backup:
                self.run_backup(backup)
            else:
                raise Exception('Device not found')
        else:
            self.run_backup()




import logging
from netmiko import NetmikoAuthenticationException, NetmikoTimeoutException

from netbox.api.exceptions import ServiceUnavailable

logger = logging.getLogger(f"netbox_config_backup")


def napalm_init(device, ip=None, extra_args={}):
    from netbox import settings

    username = settings.PLUGINS_CONFIG.get('netbox_napalm_plugin', {}).get(
        'NAPALM_USERNAME', None
    )
    password = settings.PLUGINS_CONFIG.get('netbox_napalm_plugin', {}).get(
        'NAPALM_PASSWORD', None
    )
    timeout = settings.PLUGINS_CONFIG.get('netbox_napalm_plugin', {}).get(
        'NAPALM_TIMEOUT', None
    )
    optional_args = (
        settings.PLUGINS_CONFIG.get('netbox_napalm_plugin', {})
        .get('NAPALM_ARGS', [])
        .copy()
    )

    if device and device.platform and device.platform.napalm.napalm_args is not None:
        optional_args.update(device.platform.napalm.napalm_args)
    if extra_args != {}:
        optional_args.update(extra_args)

    # Check for primary IP address from NetBox object
    if ip is not None:
        host = str(ip.address.ip)
    elif device.primary_ip and device.primary_ip is not None:
        host = str(device.primary_ip.address.ip)
    else:
        raise ServiceUnavailable("This device does not have a primary IP address")

    # Check that NAPALM is installed
    try:
        import napalm
        from napalm.base.exceptions import ModuleImportError
    except ModuleNotFoundError as e:
        if getattr(e, 'name') == 'napalm':
            raise ServiceUnavailable(
                "NAPALM is not installed. Please see the documentation for instructions."
            )
        raise e

    # Validate the configured driver
    try:
        driver = napalm.get_network_driver(device.platform.napalm.napalm_driver)
    except ModuleImportError:
        raise ServiceUnavailable(
            "NAPALM driver for platform {} not found: {}.".format(
                device.platform, device.platform.napalm.napalm_driver
            )
        )

    # Connect to the device
    d = driver(
        hostname=host,
        username=username,
        password=password,
        timeout=timeout,
        optional_args=optional_args,
    )
    try:
        d.open()
    except Exception as e:
        if isinstance(e, NetmikoAuthenticationException):
            logger.info('Authentication error')
        elif isinstance(e, NetmikoTimeoutException):
            logger.info('Connection error')
        raise ServiceUnavailable(
            "Error connecting to the device at {}: {}".format(host, e)
        )

    return d



import strawberry_django
from netbox_config_backup import filtersets, models

from netbox.graphql.filter_mixins import autotype_decorator, BaseFilterMixin

__all__ = (
    'BackupFilter',
)


@strawberry_django.filter(models.Backup, lookups=True)
@autotype_decorator(filtersets.BackupFilterSet)
class BackupFilter(BaseFilterMixin):
    pass



import logging

from django.db import models
from django.db.models import ForeignKey
from django.utils import timezone
from django.utils.translation import gettext as _

from django_rq import get_queue

from core.choices import JobStatusChoices
from netbox.models import NetBoxModel
from utilities.querysets import RestrictedQuerySet
from .abstract import BigIDModel

logger = logging.getLogger(f"netbox_config_backup")


class BackupJob(NetBoxModel):
    runner = models.ForeignKey(
        verbose_name=_('Job Run'),
        to='core.Job',
        on_delete=models.SET_NULL,
        related_name='backup_job',
        null=True,
        blank=True
    )
    backup = ForeignKey(
        to='Backup',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
        related_name='jobs',
    )
    pid = models.BigIntegerField(
        verbose_name=_('PID'),
        null=True,
        blank=True,
    )
    created = models.DateTimeField(
        auto_now_add=True
    )
    scheduled = models.DateTimeField(
        null=True,
        blank=True
    )
    started = models.DateTimeField(
        null=True,
        blank=True
    )
    completed = models.DateTimeField(
        null=True,
        blank=True
    )
    status = models.CharField(
        max_length=30,
        choices=JobStatusChoices,
        default=JobStatusChoices.STATUS_PENDING
    )
    data = models.JSONField(
        null=True,
        blank=True
    )
    job_id = models.UUIDField(
        unique=True
    )

    objects = RestrictedQuerySet.as_manager()

    def __str__(self):
        return str(self.job_id)

    @property
    def queue(self):
        return get_queue('netbox_config_backup.jobs')

    @property
    def duration(self):
        if not self.completed:
            return None

        duration = self.completed - self.started
        minutes, seconds = divmod(duration.total_seconds(), 60)

        return f"{int(minutes)} minutes, {seconds:.2f} seconds"

    def set_status(self, status):
        """
        Helper method to change the status of the job result. If the target status is terminal, the  completion
        time is also set.
        """
        self.status = status
        if status in JobStatusChoices.TERMINAL_STATE_CHOICES:
            self.completed = timezone.now()


import logging
import os
import time
import traceback

from django.db.models import Q
from django.utils import timezone

from core.choices import JobStatusChoices
from netbox.api.exceptions import ServiceUnavailable
from netbox_config_backup.models import BackupJob, Backup
from netbox_config_backup.utils.db import close_db
from netbox_config_backup.utils.configs import check_config_save_status
from netbox_config_backup.utils.napalm import napalm_init
from netbox_config_backup.utils.rq import can_backup

logger = logging.getLogger(f"netbox_config_backup")


def remove_stale_backupjobs(job: BackupJob):
    pass

def run_backup(job_id):
    close_db()
    logger.info(f'Starting backup for job {job_id}')
    try:
        job = BackupJob.objects.get(pk=job_id)
    except Exception as e:
        logger.error(f'Unable to load job {job_id}: {e}')
        logger.debug(f'\t{traceback.format_exc()}')
        raise e

    try:
        backup = Backup.objects.get(pk=job.backup.pk)
        backup.refresh_from_db()
        pid = os.getpid()

        job.status = JobStatusChoices.STATUS_PENDING
        job.pid = pid
        job.save()

        if not can_backup(backup):
            job.status = JobStatusChoices.STATUS_FAILED
            if not job.data:
                job.data = {}
            job.data.update({'error': f'Cannot backup {backup}'})
            job.full_clean()
            job.save()
            logger.warning(f'Cannot backup {backup}')
            return

        commit = None
        try:
            ip = backup.ip if backup.ip is not None else backup.device.primary_ip
        except Exception as e:
            logger.debug(f'{e}: {backup}')
            raise e

        if ip:
            try:
                d = napalm_init(backup.device, ip)
            except (TimeoutError, ServiceUnavailable):
                job.status = JobStatusChoices.STATUS_FAILED
                job.data = {'error': f'Timeout Connecting to {backup.device} with ip {ip}'}
                logger.debug = f'Timeout Connecting to {backup.device} with ip {ip}'
                job.save()
                return

            job.status = JobStatusChoices.STATUS_RUNNING
            job.started = timezone.now()
            job.save()
            try:
                status = check_config_save_status(d)
                if status is not None:
                    if status and not backup.config_status:
                        backup.config_status = status
                        backup.save()
                    elif not status and backup.config_status:
                        backup.config_status = status
                        backup.save()
                    elif not status and backup.config_status is None:
                        backup.config_status = status
                        backup.save()
                    elif status and backup.config_status is None:
                        backup.config_status = status
                        backup.save()
            except Exception as e:
                logger.error(f'{backup}: had error setting backup status: {e}')

            configs = d.get_config()
            commit = backup.set_config(configs)

            d.close()
            logger.info(f'{backup}: Backup complete')
            job.status = JobStatusChoices.STATUS_COMPLETED
            job.completed = timezone.now()
            job.save()
            remove_stale_backupjobs(job=job)
        else:
            job.status = JobStatusChoices.STATUS_FAILED
            if not job.data:
                job.data = {}
            job.data.update({'error': f'{backup}: No IP set'})
            job.full_clean()
            job.save()
            logger.debug(f'{backup}: No IP set')
    except Exception as e:
        logger.error(f'Exception in {job_id}: {e}')
        logger.info(f'\t{traceback.format_exc()}')
        if job:
            job.status = JobStatusChoices.STATUS_ERRORED
            if not job.data:
                job.data = {}
            job.data.update({'error': f'{e}'})
            job.full_clean()
            job.save()



import logging
import time
import uuid
import traceback
from datetime import timedelta
from multiprocessing import Process

from django.db.models import Q
from django.utils import timezone
from rq.job import JobStatus

from core.choices import JobStatusChoices
from netbox.jobs import JobRunner
from netbox_config_backup.backup.processing import run_backup
from netbox_config_backup.choices import StatusChoices
from netbox_config_backup.models import Backup, BackupJob
from netbox_config_backup.utils.db import close_db
from netbox_config_backup.utils.rq import can_backup

logger = logging.getLogger(f"netbox_config_backup")


class SchedulerRunner(JobRunner):
    class Meta:
        name = "The scheduler"




class BackupRunner(JobRunner):
    processes = {}

    class Meta:
        name = 'The Backup Job Runner'

    def clean_stale_jobs(self):
        jobs = BackupJob.objects.order_by('created').filter(
            status=JobStatusChoices.ENQUEUED_STATE_CHOICES,
        ).prefetch_related('device')
        scheduled = jobs.filter(status=JobStatusChoices.STATUS_SCHEDULED)
        stale = jobs.filter(scheduled__lt=timezone.now() - timedelta(minutes=30))

        for job in stale:
            if job.pid:
                pass
            job.status = JobStatusChoices.STATUS_ERRORED
            if not job.data:
                job.data = {}
            job.data.update({'error': 'Job hung'})
            job.save()
            job.refresh_from_db()
            logger.warning(f'Job {job.backup} appears stuck, deleting')

        for job in scheduled:
            if job != scheduled.filter(backup=job.backup).last():
                job.status = JobStatusChoices.STATUS_FAILED
                if not job.data:
                    job.data = {}
                job.data.update({'error': 'Process terminated'})
                job.save()

    def schedule_jobs(self, backup=None, device=None):
        if backup:
            logging.debug(f'Scheduling backup for backup: {backup}')
            backups = Backup.objects.filter(pk=backup.pk, status=StatusChoices.STATUS_ACTIVE, device__isnull=False)
        elif device:
            logging.debug(f'Scheduling backup for device: {device}')
            backups = Backup.objects.filter(device=device, status=StatusChoices.STATUS_ACTIVE, device__isnull=False)
        else:
            logging.debug(f'Scheduling all backups')
            backups = Backup.objects.filter(status=StatusChoices.STATUS_ACTIVE, device__isnull=False)

        for backup in backups:
            if can_backup(backup):
                logger.debug(f'Queuing device {backup.device} for backup')
                jobs = BackupJob.objects.filter(backup=backup, status__in=JobStatusChoices.ENQUEUED_STATE_CHOICES)
                job = jobs.last()
                if job is not None:
                    job.runner = self.job
                    job.status = JobStatusChoices.STATUS_SCHEDULED
                    job.scheduled = timezone.now()
                    job.save()
                else:
                    job = BackupJob(
                        runner=self.job,
                        backup=backup,
                        status=JobStatusChoices.STATUS_SCHEDULED,
                        scheduled=timezone.now(),
                        job_id=uuid.uuid4(),
                        data={},
                    )
                job.full_clean()
                job.save()
            else:
                jobs = BackupJob.objects.filter(backup=backup, status__in=JobStatusChoices.ENQUEUED_STATE_CHOICES)
                for job in jobs:
                    job.status = JobStatusChoices.STATUS_FAILED
                    if not job.data:
                        job.data = {}
                    job.data.update({'error': f'Cannot queue job'})
                    job.save()

    def run_processes(self):
        for job in BackupJob.objects.filter(runner=self.job, status=JobStatusChoices.STATUS_SCHEDULED):
            try:
                process = self.fork_process(job)
                process.join(1)
            except Exception as e:
                job.status = JobStatusChoices.STATUS_FAILED
                job.data['error'] = str(e)
                job.save()

    def fork_process(self, job):
        close_db()
        process = Process(target=run_backup, args=(job.pk, ), )
        data = {
            job.backup.pk: {
                'process': process,
                'backup': job.backup.pk,
                'job': job.pk
            }
        }
        self.processes.update(data)
        process.start()
        logger.debug(f'Forking process {process.pid} for {job.backup} backup')
        return process

    def handle_processes(self):
        close_db()
        for pk in list(self.processes.keys()):
            process = self.processes.get(pk, {}).get('process')
            job_pk = self.processes.get(pk, {}).get('job')
            backup = self.processes.get(pk, {}).get('backup')
            if not process.is_alive():
                logger.debug(f'Terminating process {process.pid} with job pk of {pk} for {backup}')
                process.terminate()
                del self.processes[pk]
                job = BackupJob.objects.filter(pk=job_pk).first()
                if job and job.status != JobStatusChoices.STATUS_COMPLETED:
                    job.status = JobStatusChoices.STATUS_ERRORED
                    if not job.data:
                        job.data = {}
                    job.data.update({'error': 'Process terminated'})
                    job.save()

    def run(self, backup=None, device=None, *args, **kwargs):
        try:
            self.clean_stale_jobs()
            self.schedule_jobs(backup=backup, device=device)
            self.run_processes()
            while(True):
                self.handle_processes()
                if len(self.processes) == 0:
                    return
                time.sleep(1)
        except Exception as e:
            logger.warning(f'{traceback.format_exc()}')
            logger.error(f'{e}')



from datetime import datetime
import re

from netbox_config_backup.utils.logger import get_logger
logger = get_logger()


def check_config_save_status(d):
    logger.debug(f'Switch: {d.hostname}')
    platform = {
        'ios': {
            'running': {
                'command': 'show running-config | inc ! Last configuration change',
                'regex': r'(?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+) \S+ \S+ (?P<month>\S+) (?P<day>\d+) (?P<year>\S+)(?: by \S+)?'
            },
            'startup':{
                'command': 'show startup-config | inc ! Last configuration change',
                'regex': r'(?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+) \S+ \S+ (?P<month>\S+) (?P<day>\d+) (?P<year>\S+)(?: by \S+)?'
            }
        },
        'nxos_ssh': {
            'running': {
                'command': 'show running-config | inc "!Running configuration last done at:"',
                'regex': r'(?P<month>\S+)\s+(?P<day>\d+) (?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+) (?P<year>\d+)'
            },
            'startup': {
                'command': 'show startup-config | inc "!Startup config saved at:"',
                'regex': r'(?P<month>\S+)\s+(?P<day>\d+) (?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+) (?P<year>\d+)'
            }
        },
    }

    try:
        datetimes = {
            'running': None,
            'startup': None
        }
        dates = {
            'running': None,
            'startup': None
        }
        for file in ['running', 'startup']:
            command = d.cli(commands=[platform.get(d.platform, {}).get(file, {}).get('command', '')])
            result = list(command.values()).pop()
            regex = platform.get(d.platform, {}).get(file, {}).get('regex', '')
            search = re.search(regex, result)

            if search is not None and search.groupdict():
                status = search.groupdict()
                year = status.get('year')
                month = status.get('month')
                day = f"0{int(status.get('day'))}" if int(status.get('day')) < 10 else f"{int(status.get('day'))}"
                hours = status.get('hours')
                minutes = status.get('minutes')
                seconds = status.get('seconds')

                date = f'{year}-{month}-{day} {hours}:{minutes}:{seconds}'

                dates[file] = date
                datetimes[file] = datetime.strptime(date, '%Y-%b-%d %H:%M:%S')
            else:
                logger.debug(f'\tNo {file} time found, platform: {d.platform}')

        if datetimes['running'] is None and datetimes['startup'] is not None:
            logger.debug(f'\tValid backup as booted from startup')
            return True
        elif datetimes['startup'] is None:
            logger.debug(f'\tNo startup time')
            return
        elif datetimes['running'] <= datetimes['startup']:
            logger.debug(f'\tRunning config less then startup')
            return True
        elif datetimes['running'] > datetimes['startup']:
            logger.debug(f'\tRunning config greater then startup')
            return False

    except Exception as e:

        logger.error(f'Exception when trying to check config status: {e}')


# Generated by Django 5.0.8 on 2024-10-01 23:44

import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0121_customfield_related_object_filter'),
        ('netbox_config_backup', '0017_add_job_to_backupjob'),
    ]

    operations = [
        migrations.AddField(
            model_name='backupjob',
            name='custom_field_data',
            field=models.JSONField(
                blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder
            ),
        ),
        migrations.AddField(
            model_name='backupjob',
            name='last_updated',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name='backupjob',
            name='tags',
            field=taggit.managers.TaggableManager(
                through='extras.TaggedItem', to='extras.Tag'
            ),
        ),
        migrations.AlterField(
            model_name='backupjob',
            name='id',
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False
            ),
        ),
    ]



# Generated by Django 5.0.8 on 2024-09-06 02:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0013_backup__to_netboxmodel'),
    ]

    operations = [
        migrations.AddField(
            model_name='backup',
            name='config_status',
            field=models.BooleanField(blank=True, null=True),
        ),
    ]



# Generated by Django 5.0.8 on 2024-09-12 13:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0014_backup_config_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='backup',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='backup',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
    ]



from typing import Annotated

import strawberry
import strawberry_django

from netbox.graphql.types import NetBoxObjectType
from .filters import *

from netbox_config_backup import models

__all__ = (
    'BackupType',
)


@strawberry_django.type(
    models.Backup,
    fields='__all__',
    filters=BackupFilter
)
class BackupType(NetBoxObjectType):

    name: str
    device: Annotated["DeviceType", strawberry.lazy('dcim.graphql.types')] | None
    ip: Annotated["IPAddressType", strawberry.lazy('ipam.graphql.types')] | None



from django import forms
from django.core.exceptions import ValidationError
from django.forms import CharField
from django.utils.translation import gettext as _

from core.choices import JobStatusChoices
from dcim.choices import DeviceStatusChoices
from dcim.models import Device
from ipam.models import IPAddress
from netbox.forms import NetBoxModelForm, NetBoxModelBulkEditForm
from netbox_config_backup.models import Backup, BackupJob
from utilities.forms import add_blank_choice
from utilities.forms.fields import DynamicModelChoiceField, DynamicModelMultipleChoiceField, CommentField

__all__ = (
    'BackupForm',
    'BackupJobFilterSetForm',
    'BackupFilterSetForm',
    'BackupBulkEditForm',
)

from utilities.forms.rendering import FieldSet


class BackupForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        label='Device',
        required=False,
        queryset=Device.objects.all(),
        help_text='The device this backup operates on',
        query_params={
            'status': [DeviceStatusChoices.STATUS_ACTIVE],
            'has_primary_ip': True,
        },
    )
    ip = DynamicModelChoiceField(
        label='IP Address',
        required=False,
        queryset=IPAddress.objects.all(),
        help_text='This field requires the device to be set',
        query_params={
            'device_id': '$device',
            'assigned_to_interface': True
        },
    )
    comments = CommentField()

    class Meta:
        model = Backup
        fields = ('name', 'device', 'ip', 'status', 'description', 'comments', 'config_status')

    def clean(self):
        super().clean()
        if self.cleaned_data.get('ip') and not self.cleaned_data.get('device'):
            raise ValidationError({'ip': f'Device must be set'})

        if self.cleaned_data.get('device'):
            device = self.cleaned_data.get('device')
            if not device.platform:
                raise ValidationError({'device': f'{device} has no platform set'})
            elif not hasattr(device.platform, 'napalm'):
                raise ValidationError({'device': f'{device}\'s platform ({device.platform}) has no napalm driver'})


class BackupJobFilterSetForm(forms.Form):
    model = BackupJob
    field_order = [
        'q', 'status',
    ]
    status = forms.MultipleChoiceField(
        required=False,
        choices=add_blank_choice(JobStatusChoices),
        label=_('Status')
    )


class BackupFilterSetForm(forms.Form):
    model = Backup
    field_order = [
        'q', 'name', 'device_id', 'ip'
    ]
    q = forms.CharField(
        required=False,
        widget=forms.TextInput(attrs={'placeholder': _('All Fields')}),
        label=_('Search')
    )
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        label=_('Device'),
        query_params={
            'status': [DeviceStatusChoices.STATUS_ACTIVE],
            'platform__napalm__ne': None,
            'has_primary_ip': True,
        },
    )
    ip = forms.CharField(
        required=False,
        widget=forms.TextInput(
            attrs={
                'placeholder': 'IP Address',
            }
        ),
        label=_('IP Address')
    )


class BackupBulkEditForm(NetBoxModelBulkEditForm):

    description = forms.CharField(
        label=_('Description'),
        max_length=200,
        required=False
    )
    comments = CommentField()

    model = Backup
    fieldsets = ()
    nullable_fields = ()



from typing import List

import strawberry
import strawberry_django

from .types import *


@strawberry.type(name="Query")
class BackupQuery:
    backup: BackupType = strawberry_django.field()
    backup_list: List[BackupType] = strawberry_django.field()


schema = [
    BackupQuery,
]



import logging
import sys


def get_logger():
    # Setup logging to Stdout
    formatter = logging.Formatter(f'[%(asctime)s][%(levelname)s] - %(message)s')
    stdouthandler = logging.StreamHandler(sys.stdout)
    stdouthandler.setLevel(logging.DEBUG)
    stdouthandler.setFormatter(formatter)
    logger = logging.getLogger(f"netbox_config_backup")
    logger.addHandler(stdouthandler)

    return logger



from django.urls import path

from netbox.views.generic import ObjectChangeLogView
from . import views
from .models import Backup

urlpatterns = [
    path('jobs/', views.BackupJobListView.as_view(), name='backupjob_list'),
    path('unassigned/', views.UnassignedBackupListView.as_view(), name='unassignedbackup_list'),
    path('devices/', views.BackupListView.as_view(), name='backup_list'),
    path('devices/add/', views.BackupEditView.as_view(), name='backup_add'),
    path('devices/<int:pk>/', views.BackupView.as_view(), name='backup'),
    path('devices/<int:pk>/changelog', ObjectChangeLogView.as_view(), name="backup_changelog", kwargs={'model': Backup}),
    path('devices/<int:pk>/edit/', views.BackupEditView.as_view(), name='backup_edit'),
    path('devices/<int:pk>/delete/', views.BackupDeleteView.as_view(), name='backup_delete'),
    path('devices/<int:pk>/backups/', views.BackupBackupsView.as_view(), name='backup_backups'),
    path('devices/<int:backup>/config/', views.ConfigView.as_view(), name='backup_config'),
    path('devices/<int:backup>/config/<int:current>/', views.ConfigView.as_view(), name='backup_config'),
    path('devices/<int:backup>/diff/', views.DiffView.as_view(), name='backup_diff'),
    path('devices/<int:backup>/diff/<int:current>/', views.DiffView.as_view(), name='backup_diff'),
    path('devices/<int:backup>/compliance/', views.ComplianceView.as_view(), name='backup_compliance'),
    path('devices/<int:backup>/compliance/<int:current>/', views.ComplianceView.as_view(), name='backup_compliance'),
    path('devices/edit/', views.BackupBulkEditView.as_view(), name='backup_bulk_edit'),
    path('devices/delete/', views.BackupBulkDeleteView.as_view(), name='backup_bulk_delete'),
    path('devices/<int:backup>/diff/<int:current>/<int:previous>/', views.DiffView.as_view(), name='backup_diff'),
]



from rest_framework import serializers

from dcim.api.serializers import DeviceSerializer
from ipam.api.serializers import IPAddressSerializer
from netbox.api.serializers import NetBoxModelSerializer

from netbox_config_backup.models import Backup


__all__ = (
    'BackupSerializer',
)


class BackupSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_config_backup-api:backup-detail'
    )
    device = DeviceSerializer(nested=True, required=False, allow_null=True),
    ip = IPAddressSerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = Backup
        fields = [
            'id', 'url', 'display', 'name', 'device', 'ip',
            'uuid', 'status', 'config_status',
        ]
        brief_fields = ('display', 'id', 'name', 'url')



# Generated by Django 5.0.8 on 2024-10-01 01:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0012_job_object_type_optional'),
        ('netbox_config_backup', '0016_add_pid_to_backup_job'),
    ]

    operations = [
        migrations.AddField(
            model_name='backupjob',
            name='runner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name='backup_job',
                to='core.job',
            ),
        ),
    ]



import logging

import uuid
from django.utils import timezone
from django_rq import get_queue
from rq.registry import ScheduledJobRegistry

from dcim.choices import DeviceStatusChoices
from core.choices import JobStatusChoices
from netbox_config_backup.choices import StatusChoices
from netbox_config_backup.models.jobs import BackupJob

logger = logging.getLogger(f"netbox_config_backup")


def can_backup(backup):
    logger.debug(f'Checking backup status for {backup}')
    if backup.device is None:
        logger.info(f'No device for {backup}')
        return False
    elif backup.status == StatusChoices.STATUS_DISABLED:
        logger.info(f'Backup disabled for {backup}')
        return False
    elif backup.device.status in [DeviceStatusChoices.STATUS_OFFLINE,
                                DeviceStatusChoices.STATUS_FAILED,
                                DeviceStatusChoices.STATUS_INVENTORY,
                                DeviceStatusChoices.STATUS_PLANNED]:
        logger.info(f'Backup disabled for {backup} due to device status ({backup.device.status})')
        return False
    elif (backup.ip is None and backup.device.primary_ip is None) or backup.device.platform is None or \
            hasattr(backup.device.platform, 'napalm') is False or backup.device.platform.napalm is None or \
            backup.device.platform.napalm.napalm_driver == '' or backup.device.platform.napalm.napalm_driver is None:
        if backup.ip is None and backup.device.primary_ip is None:
            logger.warning(f'Backup disabled for {backup} due to no primary IP ({backup.device.status})')
        elif backup.device.platform is None:
            logger.warning(f'Backup disabled for {backup} due to platform not set ({backup.device.status})')
        elif hasattr(backup.device.platform, 'napalm') is False or backup.device.platform.napalm is None:
            logger.warning(
                f'Backup disabled for {backup} due to platform having no napalm config ({backup.device.status})'
            )
        elif backup.device.platform.napalm.napalm_driver == '' or backup.device.platform.napalm.napalm_driver is None:
            logger.warning(f'Backup disabled for {backup} due to napalm driver not set ({backup.device.status})')
        return False

    return True



# Generated by Django 5.0.8 on 2024-09-30 21:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0015_backup_comments_backup_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='backupjob',
            name='pid',
            field=models.BigIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 5.0.9 on 2024-11-18 23:14

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0191_module_bay_rebuild'),
        ('netbox_config_backup', '0018_move_to_nbmodel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='backup',
            name='device',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name='backups',
                to='dcim.device',
            ),
        ),
    ]
